//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <vcl\registry.hpp>
#include <filectrl.hpp>
#include <sysutils.hpp>
#pragma hdrstop

#include "UninstDlg.h"
#include "Version.h"
#include "Global.h"
#include "FileMan1.h"
#include "WaitDlg.h"
#include "mixmaster.h" // MixKey
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TUninstWiz *UninstWiz;
//---------------------------------------------------------------------------
__fastcall TUninstWiz::TUninstWiz(TComponent* Owner)
  : TForm(Owner)
{
   // init some values
   Notebook->PageIndex = 0;

   AppEntries->Caption = AppName;
   MixEntries->Caption = "Mixmaster";
   Appname->Caption = AppName;//+"  "+ReleaseVer;
   Label25->Caption = "Un-Install "+AppName;

   TRegistry *reg = new TRegistry();
   mixkeyexists = reg->KeyExists(MixKey);
   appkeyexists = reg->KeyExists(QuickKey);
   if (!appkeyexists && !mixkeyexists)
   {
      delete reg;
      NextBtn->Enabled = false;
      RichEdit7->Lines->Text =
         "Neither QuickSilver nor Mixmaster are installed on this machine!\nNothing to un-install!";
      Label24->Enabled = false;
      return;
   }
   if (reg->OpenKey(MixKey,false))
   {
      mixdir = AddBackslash(reg->ReadString("MixDir"));
      reg->CloseKey();
   }
   if (reg->OpenKey(QuickKey,false))
   {
      appdir = AddBackslash(reg->ReadString("AppDir"));
      proggroup = reg->ReadString("Program Group");
      reg->CloseKey();
   }
   delete reg;

   // disable checkboxes that don't apply
   if (!mixkeyexists) TurnOff(MixEntries);
   if (!appkeyexists || FileExists(appdir+"unqs.dsk")) TurnOff(AppEntries);
   if (appkeyexists && !FileExists(appdir+"unqs.dsk")) AppEntries->Checked = true;
   if (mixkeyexists) MixEntries->Checked = true;

   if (MixInApp() || MixIsApp()) MixEntries->Enabled = false;
   if (AppInMix()) AppEntries->Checked = true;
   NextBtn->Enabled = MixEntries->Checked || AppEntries->Checked;

   AnsiString notinstalled = "not installed";
   AnsiString mixcap = mixkeyexists ? RemoveBackslash(mixdir) : notinstalled;
   AnsiString appcap = appkeyexists ? RemoveBackslash(appdir) : notinstalled;
   MixEntries->Caption = MixEntries->Caption+"    { "+mixcap.LowerCase()+" }";
   AppEntries->Caption = AppEntries->Caption+"  { "+appcap.LowerCase()+" }";

   doappfirst = AppEntries->Checked && (!domixfirst && (MixIsApp() || !AppInMix()));
   domixfirst = MixEntries->Checked && (MixIsSep() ||  MixInApp());
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::NextBtnClick(TObject *Sender)
{
   if (NextBtn->Caption=="Finish" && MsgBox("Uninstall now?",AppName,MB_OKCANCEL)==IDOK)
   {
      if (DoWaitDlg(AppName,"...uninstalling software...",this))
      {
         Notebook->PageIndex++;
         Application->ProcessMessages();
         RemoveSoftware();
         CancelBtn->Caption = "Exit";
         CancelBtn->Enabled = true;
      }
      return;
   }
   LockWindowUpdate(Handle);
   Notebook->PageIndex++;
   AnsiString Page = Notebook->ActivePage;
   if (Page=="Review") LoadReview();
   NextBtn->Caption = Page=="Review" || Page=="Finish" ? "Finish" : "Next";
   BackBtn->Enabled = Notebook->ActivePage=="Start" ? false : true;
   LockWindowUpdate(0);
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::BackBtnClick(TObject *Sender)
{
   LockWindowUpdate(Handle);
   Notebook->PageIndex--;
   AnsiString Page = Notebook->ActivePage;
   NextBtn->Caption = Page=="Review" || Page=="Finish" ? "Finish" : "Next";
   BackBtn->Enabled = Page=="Start" ? false : true;
   LockWindowUpdate(0);
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::CancelBtnClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Notebook->PageIndex==0 || (!appkeyexists && !mixkeyexists)) return;
   if (CancelBtn->Caption != "Exit" &&
       MsgBox("Quit without un-installing?","Un-Install",MB_YESNO)==IDNO)
   {
      CanClose = false;
      return;
   }
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::RemoveSoftware()
{
   RichEdit2->Clear();
   BackBtn->Enabled = false;
   NextBtn->Enabled = false;
   CancelBtn->Enabled = false;
   Application->ProcessMessages();
   if (doappfirst)
   {
      RemoveQuickSilver();
      if (MixEntries->Checked) RemoveMixmaster();
   }
   else
   if (domixfirst)
   {
      RemoveMixmaster();
      if (AppEntries->Checked) RemoveQuickSilver();
   }
   Label4->Caption = "";
   Label4->Font->Color = clBlue;
   Application->ProcessMessages();
   Label4->Caption = "Un-Installation complete!";
   RichEdit2->Lines->Add("Done!");
   Label6->Caption = "Thanks!";
   MyBeep(MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::RemoveQuickSilver()
{
   if (!AppEntries->Checked) return;
   RichEdit2->Lines->Add("Removing QuickSilver...");
   Sleep(300);
   RemoveProgGroup();
   RemoveProgFiles();
   RemoveProgEntries();
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::RemoveMixmaster()
{
   if (!MixEntries->Checked) return;
   RichEdit2->Lines->Add("Removing Mixmaster...");
   Sleep(300);
   RemoveMixFiles();
   RemoveMixEntries();
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::LoadReview()
{
   Memo1->Clear();
   Memo1->Lines->Add(AppName + (AppEntries->Checked ? " = YES" : " = NO"));
   Memo1->Lines->Add(AnsiString("Mixmaster") + (MixEntries->Checked ? " = YES" : " = NO"));
   Memo1->Perform(EM_SCROLL,SB_PAGEUP,0);
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::TurnOff(TCheckBox *box)
{
   box->Checked = false; box->Enabled = false;
//   if (box==AppEntries) ChkAppEntries->Enabled = false;
//   if (box==MixEntries) ChkMixEntries->Enabled = false;
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::RemoveProgGroup()
{
   if (!DDEClient->SetLink("PROGMAN","PROGMAN"))
   {
      MsgBox("Unable to remove "+AppName+" program group!","Error",MB_OK);
      return false;
   }
   bool ok = true;
   try {
      if (!DDEClient->ExecuteMacro(AnsiString("[DeleteGroup("+proggroup+")]").c_str(),false))
         MsgBox("ExecuteMacroLines failed!\nUnable to create Program Group!","Program Error",MB_OK);
   }
   catch (...) {
      MsgBox("Exception caught at ExecuteMacro","Exception",MB_OK);
      DDEClient->CloseLink();
      ok = false;
   }
   if (ok) {
      DDEClient->CloseLink();
   }
   RichEdit2->Lines->Add("   program group...done");
   Application->ProcessMessages();
   Sleep(300);
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::RemoveProgEntries()
{
   TRegistry *reg = new TRegistry();
   reg->DeleteKey(QuickKey);
   delete reg;
   RichEdit2->Lines->Add("   registry key...done");
   Application->ProcessMessages();
   Sleep(300);
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall DeleteQuickSilverFiles(AnsiString &pathname)
{
   if (pathname.IsEmpty() || !DirectoryExists(pathname)) return false;
   int i;
   TSearchRec sr;
   AnsiString del,fname,aname;
   pathname = AddBackslash(pathname);
   TStringList *dirs = new TStringList();
   TStringList *files = new TStringList();
   int done = FindFirst(pathname+"*.*",faAnyFile,sr);
   while (!done)
   {
      if (sr.Name != "." && sr.Name != "..")
      {
         del = pathname+sr.Name;
         if (sr.Attr!=faDirectory) files->Add(del.LowerCase());
         else dirs->Add(del.LowerCase());
      }
      done = FindNext(sr);
   }
   // remove directories
   // for (i=0; i<dirs->Count; i++)
   // {
   //   del = GetShortFileName(dirs->Strings[i]);
   //   if (!DeleteDirectory(del.c_str())) ;
   //      MsgBox("Unable to delete directory:\n"+del,"Problem",MB_OK);
   // }
   // remove files
   for (i=0; i<files->Count; i++)
   {
      del = files->Strings[i];
      fname = ExtractFileName(del).LowerCase();
      aname = ChangeFileExt(ExtractFileName(AppExe),"").LowerCase();
      if (fname==aname+DatExt ||
          fname==aname+PgpExt ||
          fname==aname+PgpExt+BakExt)
             continue;

      if (!DeleteFile(del.c_str())) ;
      // MsgBox("Unable to delete file: "+del,"Problem",MB_OK);
   }
   delete files;
   delete dirs;
   return true; //RemoveDirectory(pathname.c_str());
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::RemoveProgFiles()
{
   DeleteQuickSilverFiles(appdir);
   //DeleteDirectory(appdir);
   //SetRunOnce("Remove UninstQS","command.com /c del "+Application->ExeName);
   RichEdit2->Lines->Add("   program files...done");
   Application->ProcessMessages();
   Sleep(300);
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::RemoveMixEntries()
{
   TRegistry *reg = new TRegistry();
   reg->DeleteKey(MixKey);
   delete reg;
   RichEdit2->Lines->Add("   registry key...done");
   Application->ProcessMessages();
   Sleep(300);
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall DeleteMixmasterFiles(AnsiString &pathname)
{
   if (pathname.IsEmpty() || !DirectoryExists(pathname)) return false;
   int i;
   TSearchRec sr;
   AnsiString del;
   pathname = AddBackslash(pathname);
   TStringList *dirs = new TStringList();
   TStringList *files = new TStringList();
   int done = FindFirst(pathname+"*.*",faAnyFile,sr);
   while (!done)
   {
      if (sr.Name != "." && sr.Name != "..")
      {
         del = pathname+sr.Name;
         if (sr.Attr!=faDirectory) files->Add(del.LowerCase());
         else dirs->Add(del.LowerCase());
      }
      done = FindNext(sr);
   }
   // remove directories
   // for (i=0; i<dirs->Count; i++)
   // {
   //   del = GetShortFileName(dirs->Strings[i]);
   //   if (!DeleteDirectory(del.c_str())) ;
   //      MsgBox("Unable to delete directory:\n"+del,"Problem",MB_OK);
   // }
   // remove files
   for (i=0; i<files->Count; i++)
   {
      del = files->Strings[i];
      if (!DeleteFile(del.c_str())) ;
      // MsgBox("Unable to delete file: "+del,"Problem",MB_OK);
   }
   bool ok = true;
   // sm(dirs->Text);
   int pool = dirs->IndexOf(pathname+"pool");
   if (pool>=0)
   {
      ok = DeleteDirectory(pathname+"pool");
      dirs->Delete(pool);
   }
   if (ok && !dirs->Count)
      ok = RemoveDirectory(pathname.c_str());
   delete files;
   delete dirs;
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::RemoveMixFiles()
{
   DeleteMixmasterFiles(AddBackslash(mixdir));
   RichEdit2->Lines->Add("   program files...done");
   Application->ProcessMessages();
   Sleep(300);
   return true;
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::AppEntriesClick(TObject *Sender)
{
   if (AppEntries->Checked)
   {
      if (MixInApp() || MixIsApp()) { MixEntries->Checked=true; MixEntries->Enabled=false; }
   }
   else
   {
      if (MixIsApp() || MixInApp()) { MixEntries->Enabled=true; }
   //   if (MixInApp()) { MixEntries->Checked=false; MixEntries->Enabled=true; }
   //   if (MixIsApp()) { MixEntries->Enabled=true; }
   }
   NextBtn->Enabled = (AppEntries->Checked || MixEntries->Checked);
}
//---------------------------------------------------------------------------
void __fastcall TUninstWiz::MixEntriesClick(TObject *Sender)
{
   if (MixEntries->Checked) {
      if (AppInMix() || MixIsApp()) { AppEntries->Checked=true; AppEntries->Enabled=false; }
   }
   else
   {
      if (MixIsApp() || AppInMix()) { AppEntries->Enabled=true; }
//      if (AppInMix()) { AppEntries->Checked=false; AppEntries->Enabled=true; }
//      if (MixIsApp()) { AppEntries->Enabled=true; }
   }
   NextBtn->Enabled = (AppEntries->Checked || MixEntries->Checked);
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::MixInApp()
{
   AnsiString app,mix;
   if (appkeyexists) app = GetShortFileName(appdir).LowerCase();
   if (mixkeyexists) mix = GetShortFileName(mixdir).LowerCase();
   return !app.IsEmpty() && !mix.IsEmpty() && mix.Length()>app.Length() && mix.Pos(app)==1;
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::AppInMix()
{
   AnsiString app,mix;
   if (appkeyexists) app = GetShortFileName(appdir).LowerCase();
   if (mixkeyexists) mix = GetShortFileName(mixdir).LowerCase();
   return !app.IsEmpty() && !mix.IsEmpty() && app.Length()>mix.Length() && app.Pos(mix)==1;
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::MixIsApp()
{
   AnsiString app,mix;
   if (appkeyexists) app = GetShortFileName(appdir).LowerCase();
   if (mixkeyexists) mix = GetShortFileName(mixdir).LowerCase();
   return !app.IsEmpty() && !mix.IsEmpty() && app==mix;
}
//---------------------------------------------------------------------------
bool __fastcall TUninstWiz::MixIsSep()
{
   AnsiString app,mix;
   if (appkeyexists) app = GetShortFileName(appdir).LowerCase();
   if (mixkeyexists) mix = GetShortFileName(mixdir).LowerCase();
   return !app.IsEmpty() &&
          !mix.IsEmpty() &&
          !MixInApp() &&
          !MixIsApp() &&
          !AppInMix() &&
           app!=mix;
}
//---------------------------------------------------------------------------
